/*
 * MIT License
 *
 * Copyright (c) 2019 norangebit
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

package it.norangeb.algorithms.graph.operations

import it.norangeb.algorithms.graph.DiGraph
import org.amshove.kluent.`should be equal to`
import org.amshove.kluent.`should equal`
import org.spekframework.spek2.Spek
import org.spekframework.spek2.style.gherkin.Feature

object TopologicalOrderTest : Spek({
    Feature("topological order") {
        Scenario("topological order on DAG") {
            val dag by memoized { DiGraph() }

            Given("a dag") {
                dag.addEdge(0, 1)
                dag.addEdge(0, 2)
                dag.addEdge(0, 5)
                dag.addEdge(1, 4)
                dag.addEdge(3, 2)
                dag.addEdge(3, 4)
                dag.addEdge(3, 5)
                dag.addEdge(3, 6)
                dag.addEdge(5, 2)
                dag.addEdge(6, 0)
                dag.addEdge(6, 4)
            }

            lateinit var result: TopologicalOrder

            When("calculate the topological order") {
                result = TopologicalOrder(dag)
            }

            Then("isDag should be true") {
                result.isDag() `should be equal to` true
            }

            Then("topological order should equal listOf(3, 6, 0, 5, 2, 1, 4") {
                result.order `should equal` listOf(3, 6, 0, 5, 2, 1, 4)
            }
        }

        Scenario("topological order into cyclic graph") {
            val cyclicGraph by memoized { DiGraph() }

            Given("a cyclic graph") {
                cyclicGraph.addEdge(0, 1)
                cyclicGraph.addEdge(0, 2)
                cyclicGraph.addEdge(0, 5)
                cyclicGraph.addEdge(1, 4)
                cyclicGraph.addEdge(3, 2)
                cyclicGraph.addEdge(3, 4)
                cyclicGraph.addEdge(3, 5)
                cyclicGraph.addEdge(3, 6)
                cyclicGraph.addEdge(4, 6)
                cyclicGraph.addEdge(5, 2)
                cyclicGraph.addEdge(6, 0)
                cyclicGraph.addEdge(6, 4)
            }

            lateinit var result: TopologicalOrder

            When("compute the topological order") {
                result = TopologicalOrder(cyclicGraph)
            }

            Then("isDag should be false") {
                result.isDag() `should be equal to` false
            }

            Then("order should be null") {
                result.order `should equal` null
            }
        }
    }
})